"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.tsConfigDeclarations = exports.tsConfig = void 0;

var _options = require("../options");

var _helpers = require("../helpers");

var _defaults = require("../defaults");

// @ts-check

/**
 * @param {Partial<TsConfigParams>} paramsRaw
 */
const tsConfig = (paramsRaw = {}) => {
  const opts = (0, _options.options)();
  const {
    aliases,
    baseConfigLocation
  } = {
    aliases: opts.aliases,
    baseConfigLocation: opts.baseTsConfigLocation || './node_modules/ts-deps/lib/config/tsconfig.default.json',
    ...paramsRaw
  };
  const src = _defaults.defaults.rootDir;
  const lib = _defaults.defaults.libOutDir;
  const paths = aliases && Object.keys(aliases).reduce((acc, key) => {
    const alias = (0, _helpers.trimPathSeparator)(aliases[key]);
    return { ...acc,
      [key]: [alias],
      [`${key}/*`]: [`${alias}/*`]
    };
  }, {});
  return {
    extends: baseConfigLocation,
    compilerOptions: {
      baseUrl: '.',
      paths,
      outDir: lib
    },
    include: [src]
  };
};
/**
 * @param {Partial<TsConfigParams>} paramsRaw
 */


exports.tsConfig = tsConfig;

const tsConfigDeclarations = (paramsRaw = {}) => {
  /**
   * @type {ReturnType<typeof tsConfig> & { exclude?: string[] }}
   */
  const defConfig = tsConfig(paramsRaw);
  return { ...defConfig,
    compilerOptions: { ...defConfig.compilerOptions,
      allowJs: false,
      noEmit: false,
      declaration: true,
      emitDeclarationOnly: true
    },
    exclude: [...(Array.isArray(defConfig.exclude) && defConfig.exclude || []), _defaults.defaults.unitTestsGlob, _defaults.defaults.integrationTestsGlob]
  };
};

exports.tsConfigDeclarations = tsConfigDeclarations;
//# sourceMappingURL=tsconfig.js.map