"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.unarchiveTarGz = void 0;

var _fsExtra = require("fs-extra");

var _zlib = require("zlib");

var _tarFs = require("tar-fs");

// @ts-check

/**
 * @param {string} tar
 * @param {string} out
 * @param {import('tar-fs').ExtractOptions} [opts]
 */
const unarchiveTarGz = async (tar, out, opts) => {
  const outPath = out;
  await (0, _fsExtra.mkdir)(outPath).catch(() => Promise.resolve());
  const gunzip = (0, _zlib.createGunzip)();
  const stream = (0, _fsExtra.createReadStream)(tar).pipe(gunzip).pipe((0, _tarFs.extract)(outPath, opts), {
    end: true
  });
  return new Promise((res, rej) => {
    stream.once('end', () => {
      res();
    });
    stream.once('close', () => {
      res();
    });
    stream.once('finish', () => {
      res();
    });
    stream.once('error', err => {
      rej(err);
    });
  });
};

exports.unarchiveTarGz = unarchiveTarGz;
//# sourceMappingURL=unarchive.js.map