"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.babelBuild = babelBuild;

var _child_process = require("child_process");

var _defaults = require("../defaults");

var _fsExtra = require("fs-extra");

var _fastGlob = _interopRequireDefault(require("fast-glob"));

var _path = require("path");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// @ts-check

/**
 * @param {string[]} args All arguments
 * @param {string[]} lookupArgs Arguments we are interested in
 * @param {string[]} values Values to set
 */
const optionalArgBuilder = (args, lookupArgs, values) => {
  const contains = args.some(arg => lookupArgs.some(lookupArg => lookupArg === arg)); // if arguments already contain one of the arguments
  // we define by default, then return an empty array

  if (contains) {
    return [];
  }

  return values.map(val => val ? [lookupArgs[0], val] : [lookupArgs[0]]).reduce((acc, item) => [...acc, ...item], []);
};
/**
 * @param {BabelBuildParams} param
 */


async function babelBuild({
  commandLine,
  doNotOutputTests = true,
  logCommandLine = true,
  extensions = _defaults.defaults.extensions,
  outDir = _defaults.defaults.libOutDir,
  rootDir = _defaults.defaults.rootDir,
  unitTestsGlob = _defaults.defaults.unitTestsGlob,
  integrationTestsGlob = _defaults.defaults.integrationTestsGlob,
  copyAdditional = _defaults.defaults.copyAdditional
} = {}) {
  const exts = extensions.map(ext => `.${ext}`).join(',');
  const args = !Array.isArray(commandLine) ? process.argv.splice(2) : commandLine;
  const isHelpNeeded = args.includes('--help') || args.includes('-h');
  const skipCopying = args.includes('--no-copy-files');

  if (!isHelpNeeded && !skipCopying && copyAdditional.length > 0) {
    console.log();
    console.log('🚚  Copying files not-transformed by babel', copyAdditional.join(','), 'from', rootDir);
    /**
     * @type string[]
     */

    const files = await (0, _fastGlob.default)(copyAdditional, {
      cwd: (0, _path.join)(process.cwd(), rootDir)
    });
    await Promise.all(files.map(path => {
      const src = (0, _path.join)(rootDir, path);
      const dest = (0, _path.join)(outDir, path);
      return (0, _fsExtra.copy)(src, dest);
    }));
    console.log('✔  Successfully copied', files.length, 'files');
    console.log('');
  }
  /**
   * @param {string[]} lookup
   * @param {string[]} values
   */


  const optionalArg = (lookup, values) => optionalArgBuilder(args, lookup, values);

  const spawnArgs = isHelpNeeded ? [...args] : [rootDir, ...optionalArg(['--out-dir'], [outDir]), ...optionalArg(['-s'], ['false']), ...optionalArg(['--extensions'], [exts]), ...optionalArg(['--ignore'], doNotOutputTests ? [`${unitTestsGlob}`, `${integrationTestsGlob}`] : []), ...args];

  if (logCommandLine) {
    console.log('🚀  babel', spawnArgs.join(' '));
  }

  const babelProc = (0, _child_process.spawn)('babel', spawnArgs, {
    env: process.env,
    stdio: 'inherit',
    shell: process.platform === 'win32'
  });
  const waitForBabel = new Promise((res, _rej) => {
    babelProc.on('exit', code => {
      res(code);
    });
  });
  await waitForBabel;

  if (isHelpNeeded) {
    console.log(`
  Extra:
  --no-copy-files                             To skip copying additional files (${copyAdditional.join(',')})`);
  }
}
//# sourceMappingURL=babelBuild.js.map